@@ -8,6 +8,15 @@ import os
 import shlex
 import sys
 
+try:
+    import readline
+    HAVE_READLINE = True
+except ImportError:
+    HAVE_READLINE = False
+
+
+PYTHON3 = sys.version_info > (3, )
+
 
 class BasicParam(object):
     """ a labeled param """
@@ -129,20 +138,55 @@ def ensure_params(*params):
     return partial(ensure_params_with_parser, parser)
 
 
+MAX_OUTPUT = 1 << 20
+
+
 class AugumentedCmd(cmd.Cmd):
     """ extends cmd.Cmd """
     curdir = "/"
 
-    def __init__(self, hist_file_name=None, setup_readline=True):
+    def __init__(self, hist_file_name=None, setup_readline=True, output_io=sys.stdout):
         cmd.Cmd.__init__(self)
 
+        self._output = output_io
+        self._last_output = ""
+
         if setup_readline:
             self._setup_readline(hist_file_name)
 
+        # special commands dispatch map
+        self._special_commands = {
+            "!!": self.run_last_command,
+            "$?": self.echo_last_output,
+        }
+
+    @property
+    def output(self):
+        """ the io output object """
+        return self._output
+
+    def do_output(self, fmt_str, *params):
+        """ MAX_OUTPUT chars of the last output is available via $? """
+        if PYTHON3:
+            fmt_str = str(fmt_str)
+        out = fmt_str % params
+        self._last_output = out if len(out) < MAX_OUTPUT else out[:MAX_OUTPUT]
+        print(out, file=self._output)
+
     def default(self, line):
         args = shlex.split(line)
         if len(args) > 0 and not args[0].startswith("#"):  # ignore commented lines, ala Bash
-            print("Unknown command: %s" % (args[0]))
+            cmd = self._special_commands.get(args[0])
+            if cmd:
+                cmd(args[1:])
+            else:
+                print("Unknown command: %s" % (args[0]))
+
+    def run_last_command(self, *args):
+        self.onecmd(self.last_command)
+
+    def echo_last_output(self, *args):
+        print(self._last_output, file=self._output)
 
     def emptyline(self):
         pass
@@ -195,13 +239,16 @@ class AugumentedCmd(cmd.Cmd):
     def state(self):
         return ""
 
-    def _setup_readline(self, hist_file_name):
-        try:
-            import readline, atexit
-        except ImportError:
-            return
+    @property
+    def last_command(self):
+        if not HAVE_READLINE:
+            return ""
 
-        if hist_file_name is None:
+        cur_size = readline.get_current_history_length()
+        return readline.get_history_item(cur_size - 1)
+
+    def _setup_readline(self, hist_file_name):
+        if not HAVE_READLINE or hist_file_name is None:
             return
 
         path = os.path.join(os.environ["HOME"], hist_file_name)
@@ -209,4 +256,6 @@ class AugumentedCmd(cmd.Cmd):
             readline.read_history_file(path)
         except IOError:
             pass
+
+        import atexit
         atexit.register(readline.write_history_file, path)
