@@ -630,12 +630,16 @@ class Shell(AugumentedCmd):
         server=('127.0.0.1', 2181)
         """
         fmt_str = """state=%s
+sessionid=%s
+protocol_version=%d
 xid=%d
 last_zxid=%d
 timeout=%d
 server=%s"""
         self.do_output(fmt_str,
                        self._zk.client_state,
+                       self._zk.sessionid,
+                       self._zk.protocol_version,
                        self._zk.xid,
                        self._zk.last_zxid,
                        self._zk.session_timeout,
@@ -647,7 +651,7 @@ server=%s"""
         try:
             self.do_output(self._zk.mntr(host))
         except AugumentedClient.CmdFailed as ex:
-            self.do_output(ex)
+            self.do_output(str(ex))
 
     @ensure_params(Optional("host"))
     def do_cons(self, params):
@@ -655,7 +659,7 @@ server=%s"""
         try:
             self.do_output(self._zk.cons(host))
         except AugumentedClient.CmdFailed as ex:
-            self.do_output(ex)
+            self.do_output(str(ex))
 
     @ensure_params(Optional("host"))
     def do_dump(self, params):
@@ -663,7 +667,7 @@ server=%s"""
         try:
             self.do_output(self._zk.dump(host))
         except AugumentedClient.CmdFailed as ex:
-            self.do_output(ex)
+            self.do_output(str(ex))
 
     @connected
     @ensure_params(Required("path"))
